local dewdrop = AceLibrary("Dewdrop-2.0")
local L = AceLibrary("AceLocale-2.2"):new("SpeedFu")

local Astrolabe = DongleStub("Astrolabe-0.4")
local GetTime = GetTime
local format = string.format
local floor = math.floor

SpeedFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceConsole-2.0", "AceDB-2.0")
SpeedFu.hasIcon = "Interface\\Icons\\Ability_Rogue_Sprint.blp"

local optionsTable = {
	type = 'group',
	args = {
		api = {
			order = 2,
			type = 'toggle',
			name = L["MENU_API"],
			desc = L["TOOLTIP_API"],
			set = "ToggleAPI",
			get = "GetAPI",
		},
		digit = {
			order = 3,
			type = 'toggle',
			name = L["MENU_DIGIT"],
			desc = L["TOOLTIP_DIGIT"],
			set = "ToggleDigit",
			get = "GetDigit",
		},
	}
}

SpeedFu.blizzardTooltip = true
SpeedFu.OnMenuRequest = optionsTable
SpeedFu:RegisterChatCommand( { "/speedfu" }, optionsTable )

local FREQUENCY = 0.5
local frame

function SpeedFu:OnInitialize()
	if not frame then frame = CreateFrame("Frame") end
	frame:Hide()
	frame:SetScript("OnUpdate", self.UpdateSpeed)
	
	self:RegisterDB("FuBar_SpeedDB")
	
	self.vars = {}
	self.text = "???%"
end

function SpeedFu:OnEnable()
	frame:Show()
end

function SpeedFu:OnDisable()
	frame:Hide()
end

-- This will be called from FuBar.
function SpeedFu:OnTextUpdate()
	self:SetText(self.text)
end

function SpeedFu:ToggleAPI()
	self.db.profile.api = not self.db.profile.api
	if self.db.profile.api then
		self.vars.lastPos = nil
	end
end

function SpeedFu:GetAPI()
	return self.db.profile.api
end

function SpeedFu:ToggleDigit()
	self.db.profile.digit = not self.db.profile.digit
end

function SpeedFu:GetDigit()
	return self.db.profile.digit
end

local difference = 0
function SpeedFu.UpdateSpeed(frame,elapsed)
	difference = difference + elapsed
	if difference < FREQUENCY then
		return
	end
	
	local self = SpeedFu

	local speed
	
	if self.db.profile.api then
		speed = floor( ( GetUnitSpeed("player") / 7 * 100 ) + 0.5 )
	else
		if not self.vars.lastPos then
			local C, Z, x, y = Astrolabe:GetCurrentPlayerPosition()
			if C and Z and x and y then
				self.vars.lastTime = GetTime()
				self.vars.CurrPos = {}
				self.vars.lastPos = { C = C, Z = Z, x = x, y = y }
			else
				return
			end
		end
	
		self.vars.CurrTime = GetTime()
		self.vars.CurrPos.C, self.vars.CurrPos.Z, self.vars.CurrPos.x, self.vars.CurrPos.y = Astrolabe:GetCurrentPlayerPosition()
		
		local dist = Astrolabe:ComputeDistance(self.vars.lastPos.C, self.vars.lastPos.Z, self.vars.lastPos.x, self.vars.lastPos.y, self.vars.CurrPos.C, self.vars.CurrPos.Z, self.vars.CurrPos.x, self.vars.CurrPos.y) or 0
		local delta = self.vars.CurrTime - self.vars.lastTime
		
		if self.vars.CurrPos.x and self.vars.CurrPos.y and delta >= 0.5 then
			speed = floor( ( dist / delta / 7 * 100 ) + 0.5 )
			
			self.vars.lastTime = self.vars.CurrTime
			self.vars.lastPos.C = self.vars.CurrPos.C
			self.vars.lastPos.Z = self.vars.CurrPos.Z
			self.vars.lastPos.x = self.vars.CurrPos.x
			self.vars.lastPos.y = self.vars.CurrPos.y
		end
	end
	
	local text
	if not speed then
		--text = "???%"
	else
		if self.db.profile.digit then
			text = format("%03d%%", speed)
		else
			text = speed .. "%"
		end
		if speed >= 101 then
			text = format("|cff00ff00%s|r", text)
		elseif speed <= 99 then
			text = format("|cffff0000%s|r", text)
		else
			text = format("|cffffffff%s|r", text)
		end
	end
	
	self.text = text or self.text
	
	self:Update()
	
	difference = 0
end
